# Buildah Module

The Buildah module provides functions for working with containers and images using the Buildah tool. Buildah helps you create and manage container images.

## Builder Pattern

The Buildah module now supports a Builder pattern, which provides a more intuitive and flexible way to work with containers and images.

### Creating a Builder

```rhai
// Create a builder with a name and base image
let builder = bah_new("my-container", "alpine:latest");

// Access builder properties
let container_id = builder.container_id;
let name = builder.name;
let image = builder.image;
```

### Builder Methods

The Builder object provides the following methods:

- `run(command)`: Run a command in the container
- `run_with_isolation(command, isolation)`: Run a command with specified isolation
- `copy(source, dest)`: Copy files into the container
- `add(source, dest)`: Add files into the container
- `commit(image_name)`: Commit the container to an image
- `remove()`: Remove the container
- `reset()`: Remove the container and clear the container_id
- `config(options)`: Configure container metadata
- `images()`: List images in local storage
- `image_remove(image)`: Remove an image
- `image_pull(image, tls_verify)`: Pull an image from a registry
- `image_push(image, destination, tls_verify)`: Push an image to a registry
- `image_tag(image, new_name)`: Add a tag to an image
- `build(tag, context_dir, file, isolation)`: Build an image from a Dockerfile

### Example

```rhai
// Create a builder
let builder = bah_new("my-container", "alpine:latest");

// Reset the builder to remove any existing container
builder.reset();

// Create a new container
builder = bah_new("my-container", "alpine:latest");

// Run a command
let result = builder.run("echo 'Hello from container'");
println(`Command output: ${result.stdout}`);

// Add a file
file_write("test_file.txt", "Test content");
builder.add("test_file.txt", "/");

// Commit to an image
builder.commit("my-custom-image:latest");

// Clean up
builder.remove();
delete("test_file.txt");
```

## Image Information

### Image Properties

When working with images, you can access the following information:

- `id`: The unique identifier for the image
- `names`: A list of names/tags for the image
- `name`: The primary name of the image, or `<none>` if the image has no names
- `size`: The size of the image
- `created`: When the image was created

## Builder Methods

### `bah_new(name, image)`

Creates a new Builder object for working with a container.

**Parameters:**
- `name` (string): The name to give the container
- `image` (string): The name or ID of the image to create the container from

**Returns:** A Builder object if successful.

**Example:**
```rhai
// Create a new Builder
let builder = bah_new("my-container", "alpine:latest");
```

**Notes:**
- If a container with the given name already exists, it will be reused instead of creating a new one
- The Builder object provides methods for working with the container

### `reset()`

Resets a Builder by removing the container and clearing the container_id. This allows you to start fresh with the same Builder object.

**Returns:** Nothing.

**Example:**
```rhai
// Create a Builder
let builder = bah_new("my-container", "alpine:latest");

// Reset the Builder to remove the container
builder.reset();

// Create a new container with the same name
builder = bah_new("my-container", "alpine:latest");
```
